/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.block;

import net.minecraft.block.Block;
import net.minecraft.block.BlockColored;
import net.minecraft.block.BlockStainedGlass;
import net.minecraft.block.BlockStainedGlassPane;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.util.EnumActionResult;

import buildcraft.api.blocks.CustomPaintHelper;
import buildcraft.api.blocks.ICustomPaintHandler;

public class VanillaPaintHandlers {

    public static void fmlInit() {
        registerDoubleTypedHandler(Blocks.field_150359_w, Blocks.field_150399_cn, BlockStainedGlass.field_176547_a);
        registerDoubleTypedHandler(Blocks.field_150410_aZ, Blocks.field_150397_co, BlockStainedGlassPane.field_176245_a);
        registerDoubleTypedHandler(Blocks.field_150405_ch, Blocks.field_150406_ce, BlockColored.field_176581_a);
    }

    private static void registerDoubleTypedHandler(Block clear, Block dyed, IProperty<EnumDyeColor> colourProp) {
        ICustomPaintHandler handler = createDoubleTypedPainter(clear, dyed, colourProp);
        CustomPaintHelper.INSTANCE.registerHandler(clear, handler);
        CustomPaintHelper.INSTANCE.registerHandler(dyed, handler);
    }

    public static ICustomPaintHandler createDoubleTypedPainter(Block clear, Block dyed, IProperty<EnumDyeColor> colourProp) {
        return (world, pos, state, hitPos, hitSide, to) -> {
            if (state.getBlock() == clear) {
                // We are currently clear
                if (to == null) {
                    return EnumActionResult.FAIL;
                }
                IBlockState painted = dyed.getDefaultState().withProperty(colourProp, to);
                world.setBlockState(pos, painted);
                return EnumActionResult.SUCCESS;
            } else if (state.getBlock() == dyed) {
                if (to == state.getValue(colourProp)) {
                    return EnumActionResult.FAIL;
                }
                if (to == null) {
                    state = clear.getDefaultState();
                } else {
                    state = state.withProperty(colourProp, to);
                }
                world.setBlockState(pos, state);
                return EnumActionResult.SUCCESS;
            }
            return EnumActionResult.PASS;
        };
    }
}
